@@ -7,44 +7,51 @@ Racine, Jeff. (2008) "Nonparametric Econometrics: A Primer," Foundation and
     Trends in Econometrics: Vol 3: No 1, pp1-88.
     http://dx.doi.org/10.1561/0800000009
 
-http://en.wikipedia.org/wiki/Kernel_%28statistics%29
+https://en.wikipedia.org/wiki/Kernel_%28statistics%29
 
 Silverman, B.W.  Density Estimation for Statistics and Data Analysis.
 """
-from __future__ import absolute_import, print_function, division
-from statsmodels.compat.python import range
 import numpy as np
 from scipy import integrate, stats
+
 from statsmodels.sandbox.nonparametric import kernels
 from statsmodels.tools.decorators import cache_readonly
+from statsmodels.tools.validation import array_like, float_like
+
 from . import bandwidths
-from .kdetools import (forrt, revrt, silverman_transform)
+from .kdetools import forrt, revrt, silverman_transform
 from .linbin import fast_linbin
 
-#### Kernels Switch for estimators ####
+# Kernels Switch for estimators
+
+kernel_switch = dict(
+    gau=kernels.Gaussian,
+    epa=kernels.Epanechnikov,
+    uni=kernels.Uniform,
+    tri=kernels.Triangular,
+    biw=kernels.Biweight,
+    triw=kernels.Triweight,
+    cos=kernels.Cosine,
+    cos2=kernels.Cosine2,
+    tric=kernels.Tricube
+)
 
-kernel_switch = dict(gau=kernels.Gaussian, epa=kernels.Epanechnikov,
-                    uni=kernels.Uniform, tri=kernels.Triangular,
-                    biw=kernels.Biweight, triw=kernels.Triweight,
-                    cos=kernels.Cosine, cos2=kernels.Cosine2)
 
 def _checkisfit(self):
     try:
         self.density
-    except:
+    except Exception:
         raise ValueError("Call fit to fit the density first")
 
 
-#### Kernel Density Estimator Class ###
-
-
+# Kernel Density Estimator Class
 class KDEUnivariate(object):
     """
     Univariate Kernel Density Estimator.
 
     Parameters
     ----------
-    endog : array-like
+    endog : array_like
         The variable for which the density estimate is desired.
 
     Notes
@@ -73,14 +80,22 @@ class KDEUnivariate(object):
     >>> dens.fit()
     >>> plt.plot(dens.cdf)
     >>> plt.show()
-
     """
 
     def __init__(self, endog):
-        self.endog = np.asarray(endog)
+        self.endog = array_like(endog, "endog", ndim=1, contiguous=True)
 
-    def fit(self, kernel="gau", bw="normal_reference", fft=True, weights=None,
-            gridsize=None, adjust=1, cut=3, clip=(-np.inf, np.inf)):
+    def fit(
+        self,
+        kernel="gau",
+        bw="normal_reference",
+        fft=True,
+        weights=None,
+        gridsize=None,
+        adjust=1,
+        cut=3,
+        clip=(-np.inf, np.inf),
+    ):
         """
         Attach the density estimate to the KDEUnivariate class.
 
@@ -97,17 +112,23 @@ class KDEUnivariate(object):
             - "triw" for triweight
             - "uni" for uniform
 
-        bw : str, float
+        bw : str, float, callable
             The bandwidth to use. Choices are:
 
             - "scott" - 1.059 * A * nobs ** (-1/5.), where A is
-              `min(std(X),IQR/1.34)`
+              `min(std(x),IQR/1.34)`
             - "silverman" - .9 * A * nobs ** (-1/5.), where A is
-              `min(std(X),IQR/1.34)`
+              `min(std(x),IQR/1.34)`
             - "normal_reference" - C * A * nobs ** (-1/5.), where C is
               calculated from the kernel. Equivalent (up to 2 dp) to the
               "scott" bandwidth for gaussian kernels. See bandwidths.py
-            - If a float is given, it is the bandwidth.
+            - If a float is given, its value is used as the bandwidth.
+            - If a callable is given, it's return value is used.
+              The callable should take exactly two parameters, i.e.,
+              fn(x, kern), and return a float, where:
+
+              * x - the clipped input data
+              * kern - the kernel instance used
 
         fft : bool
             Whether or not to use FFT. FFT implementation is more
@@ -115,19 +136,26 @@ class KDEUnivariate(object):
             is implemented. If FFT is False, then a 'nobs' x 'gridsize'
             intermediate array is created.
         gridsize : int
-            If gridsize is None, max(len(X), 50) is used.
+            If gridsize is None, max(len(x), 50) is used.
         cut : float
             Defines the length of the grid past the lowest and highest values
-            of X so that the kernel goes to zero. The end points are
-            -/+ cut*bw*{min(X) or max(X)}
+            of x so that the kernel goes to zero. The end points are
+            -/+ cut*bw*{min(x) or max(x)}
         adjust : float
             An adjustment factor for the bw. Bandwidth becomes bw * adjust.
+
+        Returns
+        -------
+        KDEUnivariate
+            The instance fit,
         """
-        try:
-            bw = float(bw)
-            self.bw_method = "user-given"
-        except:
+        if isinstance(bw, str):
             self.bw_method = bw
+        else:
+            self.bw_method = "user-given"
+            if not callable(bw):
+                bw = float_like(bw, "bw")
+
         endog = self.endog
 
         if fft:
@@ -137,23 +165,38 @@ class KDEUnivariate(object):
             if weights is not None:
                 msg = "Weights are not implemented for fft"
                 raise NotImplementedError(msg)
-            density, grid, bw = kdensityfft(endog, kernel=kernel, bw=bw,
-                    adjust=adjust, weights=weights, gridsize=gridsize,
-                    clip=clip, cut=cut)
+            density, grid, bw = kdensityfft(
+                endog,
+                kernel=kernel,
+                bw=bw,
+                adjust=adjust,
+                weights=weights,
+                gridsize=gridsize,
+                clip=clip,
+                cut=cut,
+            )
         else:
-            density, grid, bw = kdensity(endog, kernel=kernel, bw=bw,
-                    adjust=adjust, weights=weights, gridsize=gridsize,
-                    clip=clip, cut=cut)
+            density, grid, bw = kdensity(
+                endog,
+                kernel=kernel,
+                bw=bw,
+                adjust=adjust,
+                weights=weights,
+                gridsize=gridsize,
+                clip=clip,
+                cut=cut,
+            )
         self.density = density
         self.support = grid
         self.bw = bw
-        self.kernel = kernel_switch[kernel](h=bw) # we instantiate twice,
-                                                # should this passed to funcs?
+        self.kernel = kernel_switch[kernel](h=bw)  # we instantiate twice,
+        # should this passed to funcs?
         # put here to ensure empty cache after re-fit with new options
         self.kernel.weights = weights
         if weights is not None:
             self.kernel.weights /= weights.sum()
         self._cache = {}
+        return self
 
     @cache_readonly
     def cdf(self):
@@ -166,18 +209,22 @@ class KDEUnivariate(object):
         """
         _checkisfit(self)
         kern = self.kernel
-        if kern.domain is None: # TODO: test for grid point at domain bound
-            a,b = -np.inf,np.inf
+        if kern.domain is None:  # TODO: test for grid point at domain bound
+            a, b = -np.inf, np.inf
         else:
-            a,b = kern.domain
-        func = lambda x,s: kern.density(s,x)
+            a, b = kern.domain
+
+        def func(x, s):
+            return kern.density(s, x)
 
         support = self.support
-        support = np.r_[a,support]
+        support = np.r_[a, support]
         gridsize = len(support)
         endog = self.endog
-        probs = [integrate.quad(func, support[i - 1], support[i],
-                    args=endog)[0] for i in range(1, gridsize)]
+        probs = [
+            integrate.quad(func, support[i - 1], support[i], args=endog)[0]
+            for i in range(1, gridsize)
+        ]
         return np.cumsum(probs)
 
     @cache_readonly
@@ -188,7 +235,6 @@ class KDEUnivariate(object):
         Notes
         -----
         Will not work if fit has not been called.
-
         """
         _checkisfit(self)
         return -np.log(self.sf)
@@ -217,9 +263,9 @@ class KDEUnivariate(object):
         """
         _checkisfit(self)
 
-        def entr(x,s):
-            pdf = kern.density(s,x)
-            return pdf*np.log(pdf+1e-12)
+        def entr(x, s):
+            pdf = kern.density(s, x)
+            return pdf * np.log(pdf + 1e-12)
 
         kern = self.kernel
 
@@ -228,7 +274,7 @@ class KDEUnivariate(object):
         else:
             a, b = -np.inf, np.inf
         endog = self.endog
-        #TODO: below could run into integr problems, cf. stats.dist._entropy
+        # TODO: below could run into integr problems, cf. stats.dist._entropy
         return -integrate.quad(entr, a, b, args=(endog,))[0]
 
     @cache_readonly
@@ -247,27 +293,35 @@ class KDEUnivariate(object):
 
     def evaluate(self, point):
         """
-        Evaluate density at a single point.
+        Evaluate density at a point or points.
 
         Parameters
         ----------
-        point : float
-            Point at which to evaluate the density.
+        point : {float, ndarray}
+            Point(s) at which to evaluate the density.
         """
         _checkisfit(self)
         return self.kernel.density(self.endog, point)
 
 
-#### Kernel Density Estimator Functions ####
-
-def kdensity(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None,
-             adjust=1, clip=(-np.inf, np.inf), cut=3, retgrid=True):
+# Kernel Density Estimator Functions
+def kdensity(
+    x,
+    kernel="gau",
+    bw="normal_reference",
+    weights=None,
+    gridsize=None,
+    adjust=1,
+    clip=(-np.inf, np.inf),
+    cut=3,
+    retgrid=True,
+):
     """
     Rosenblatt-Parzen univariate kernel density estimator.
 
     Parameters
     ----------
-    X : array-like
+    x : array_like
         The variable for which the density estimate is desired.
     kernel : str
         The Kernel to be used. Choices are
@@ -278,32 +332,46 @@ def kdensity(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None
         - "tri" for triangular
         - "triw" for triweight
         - "uni" for uniform
-    bw : str, float
-        "scott" - 1.059 * A * nobs ** (-1/5.), where A is min(std(X),IQR/1.34)
-        "silverman" - .9 * A * nobs ** (-1/5.), where A is min(std(X),IQR/1.34)
-        If a float is given, it is the bandwidth.
+    bw : str, float, callable
+        The bandwidth to use. Choices are:
+
+        - "scott" - 1.059 * A * nobs ** (-1/5.), where A is
+          `min(std(x),IQR/1.34)`
+        - "silverman" - .9 * A * nobs ** (-1/5.), where A is
+          `min(std(x),IQR/1.34)`
+        - "normal_reference" - C * A * nobs ** (-1/5.), where C is
+          calculated from the kernel. Equivalent (up to 2 dp) to the
+          "scott" bandwidth for gaussian kernels. See bandwidths.py
+        - If a float is given, its value is used as the bandwidth.
+        - If a callable is given, it's return value is used.
+          The callable should take exactly two parameters, i.e.,
+          fn(x, kern), and return a float, where:
+
+          * x - the clipped input data
+          * kern - the kernel instance used
+
     weights : array or None
-        Optional  weights. If the X value is clipped, then this weight is
+        Optional  weights. If the x value is clipped, then this weight is
         also dropped.
     gridsize : int
-        If gridsize is None, max(len(X), 50) is used.
+        If gridsize is None, max(len(x), 50) is used.
     adjust : float
         An adjustment factor for the bw. Bandwidth becomes bw * adjust.
     clip : tuple
-        Observations in X that are outside of the range given by clip are
-        dropped. The number of observations in X is then shortened.
+        Observations in x that are outside of the range given by clip are
+        dropped. The number of observations in x is then shortened.
     cut : float
-        Defines the length of the grid past the lowest and highest values of X
+        Defines the length of the grid past the lowest and highest values of x
         so that the kernel goes to zero. The end points are
-        -/+ cut*bw*{min(X) or max(X)}
+        -/+ cut*bw*{min(x) or max(x)}
     retgrid : bool
         Whether or not to return the grid over which the density is estimated.
 
     Returns
     -------
-    density : array
+    density : ndarray
         The densities estimated at the grid points.
-    grid : array, optional
+    grid : ndarray, optional
         The grid points at which the density is estimated.
 
     Notes
@@ -311,16 +379,16 @@ def kdensity(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None
     Creates an intermediate (`gridsize` x `nobs`) array. Use FFT for a more
     computationally efficient version.
     """
-    X = np.asarray(X)
-    if X.ndim == 1:
-        X = X[:, None]
-    clip_x = np.logical_and(X > clip[0], X < clip[1])
-    X = X[clip_x]
+    x = np.asarray(x)
+    if x.ndim == 1:
+        x = x[:, None]
+    clip_x = np.logical_and(x > clip[0], x < clip[1])
+    x = x[clip_x]
 
-    nobs = len(X) # after trim
+    nobs = len(x)  # after trim
 
     if gridsize is None:
-        gridsize = max(nobs,50) # don't need to resize if no FFT
+        gridsize = max(nobs, 50)  # do not need to resize if no FFT
 
         # handle weights
     if weights is None:
@@ -331,7 +399,7 @@ def kdensity(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None
         weights = np.asarray(weights)
 
         if len(weights) != len(clip_x):
-            msg = "The length of the weights must be the same as the given X."
+            msg = "The length of the weights must be the same as the given x."
             raise ValueError(msg)
         weights = weights[clip_x.squeeze()]
         q = weights.sum()
@@ -339,48 +407,66 @@ def kdensity(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None
     # Get kernel object corresponding to selection
     kern = kernel_switch[kernel]()
 
-    # if bw is None, select optimal bandwidth for kernel
-    try:
-        bw = float(bw)
-    except:
-        bw = bandwidths.select_bandwidth(X, bw, kern)
+    if callable(bw):
+        bw = float(bw(x, kern))
+        # user passed a callable custom bandwidth function
+    elif isinstance(bw, str):
+        bw = bandwidths.select_bandwidth(x, bw, kern)
+        # will cross-val fit this pattern?
+    else:
+        bw = float_like(bw, "bw")
+
     bw *= adjust
 
-    a = np.min(X, axis=0) - cut * bw
-    b = np.max(X, axis=0) + cut * bw
+    a = np.min(x, axis=0) - cut * bw
+    b = np.max(x, axis=0) + cut * bw
     grid = np.linspace(a, b, gridsize)
 
-    k = (X.T - grid[:, None])/bw  # uses broadcasting to make a gridsize x nobs
+    k = (
+        x.T - grid[:, None]
+    ) / bw  # uses broadcasting to make a gridsize x nobs
 
     # set kernel bandwidth
     kern.seth(bw)
 
     # truncate to domain
-    if kern.domain is not None: # won't work for piecewise kernels like parzen
+    if (
+        kern.domain is not None
+    ):  # will not work for piecewise kernels like parzen
         z_lo, z_high = kern.domain
         domain_mask = (k < z_lo) | (k > z_high)
-        k = kern(k) # estimate density
+        k = kern(k)  # estimate density
         k[domain_mask] = 0
     else:
-        k = kern(k) # estimate density
+        k = kern(k)  # estimate density
 
-    k[k < 0] = 0 # get rid of any negative values, do we need this?
+    k[k < 0] = 0  # get rid of any negative values, do we need this?
 
-    dens = np.dot(k, weights)/(q*bw)
+    dens = np.dot(k, weights) / (q * bw)
 
     if retgrid:
         return dens, grid, bw
     else:
         return dens, bw
 
-def kdensityfft(X, kernel="gau", bw="normal_reference", weights=None, gridsize=None,
-                adjust=1, clip=(-np.inf, np.inf), cut=3, retgrid=True):
+
+def kdensityfft(
+    x,
+    kernel="gau",
+    bw="normal_reference",
+    weights=None,
+    gridsize=None,
+    adjust=1,
+    clip=(-np.inf, np.inf),
+    cut=3,
+    retgrid=True,
+):
     """
     Rosenblatt-Parzen univariate kernel density estimator
 
     Parameters
     ----------
-    X : array-like
+    x : array_like
         The variable for which the density estimate is desired.
     kernel : str
         ONLY GAUSSIAN IS CURRENTLY IMPLEMENTED.
@@ -392,39 +478,53 @@ def kdensityfft(X, kernel="gau", bw="normal_reference", weights=None, gridsize=N
         "par" for Parzen
         "rect" for rectangular
         "tri" for triangular
-    bw : str, float
-        "scott" - 1.059 * A * nobs ** (-1/5.), where A is min(std(X),IQR/1.34)
-        "silverman" - .9 * A * nobs ** (-1/5.), where A is min(std(X),IQR/1.34)
-        If a float is given, it is the bandwidth.
+    bw : str, float, callable
+        The bandwidth to use. Choices are:
+
+        - "scott" - 1.059 * A * nobs ** (-1/5.), where A is
+          `min(std(x),IQR/1.34)`
+        - "silverman" - .9 * A * nobs ** (-1/5.), where A is
+          `min(std(x),IQR/1.34)`
+        - "normal_reference" - C * A * nobs ** (-1/5.), where C is
+          calculated from the kernel. Equivalent (up to 2 dp) to the
+          "scott" bandwidth for gaussian kernels. See bandwidths.py
+        - If a float is given, its value is used as the bandwidth.
+        - If a callable is given, it's return value is used.
+          The callable should take exactly two parameters, i.e.,
+          fn(x, kern), and return a float, where:
+
+          * x - the clipped input data
+          * kern - the kernel instance used
+
     weights : array or None
         WEIGHTS ARE NOT CURRENTLY IMPLEMENTED.
-        Optional  weights. If the X value is clipped, then this weight is
+        Optional  weights. If the x value is clipped, then this weight is
         also dropped.
     gridsize : int
-        If gridsize is None, min(len(X), 512) is used. Note that the provided
+        If gridsize is None, min(len(x), 512) is used. Note that the provided
         number is rounded up to the next highest power of 2.
     adjust : float
         An adjustment factor for the bw. Bandwidth becomes bw * adjust.
         clip : tuple
-        Observations in X that are outside of the range given by clip are
-        dropped. The number of observations in X is then shortened.
+        Observations in x that are outside of the range given by clip are
+        dropped. The number of observations in x is then shortened.
     cut : float
-        Defines the length of the grid past the lowest and highest values of X
+        Defines the length of the grid past the lowest and highest values of x
         so that the kernel goes to zero. The end points are
-        -/+ cut*bw*{X.min() or X.max()}
+        -/+ cut*bw*{x.min() or x.max()}
     retgrid : bool
         Whether or not to return the grid over which the density is estimated.
 
     Returns
     -------
-    density : array
+    density : ndarray
         The densities estimated at the grid points.
-    grid : array, optional
+    grid : ndarray, optional
         The grid points at which the density is estimated.
 
     Notes
     -----
-    Only the default kernel is implemented. Weights aren't implemented yet.
+    Only the default kernel is implemented. Weights are not implemented yet.
     This follows Silverman (1982) with changes suggested by Jones and Lotwick
     (1984). However, the discretization step is replaced by linear binning
     of Fan and Marron (1994). This should be extended to accept the parts
@@ -443,107 +543,68 @@ def kdensityfft(X, kernel="gau", bw="normal_reference", weights=None, gridsize=N
         the Fast Fourier Transform. Journal of the Royal Statistical Society.
         Series C. 31.2, 93-9.
     """
-    X = np.asarray(X)
-    X = X[np.logical_and(X > clip[0], X < clip[1])] # won't work for two columns.
-                                                # will affect underlying data?
+    x = np.asarray(x)
+    # will not work for two columns.
+    x = x[np.logical_and(x > clip[0], x < clip[1])]
 
     # Get kernel object corresponding to selection
     kern = kernel_switch[kernel]()
 
-    try:
-        bw = float(bw)
-    except:
-        bw = bandwidths.select_bandwidth(X, bw, kern) # will cross-val fit this pattern?
+    if callable(bw):
+        bw = float(bw(x, kern))
+        # user passed a callable custom bandwidth function
+    elif isinstance(bw, str):
+        # if bw is None, select optimal bandwidth for kernel
+        bw = bandwidths.select_bandwidth(x, bw, kern)
+        # will cross-val fit this pattern?
+    else:
+        bw = float_like(bw, "bw")
+
     bw *= adjust
 
-    nobs = len(X) # after trim
+    nobs = len(x)  # after trim
 
     # 1 Make grid and discretize the data
     if gridsize is None:
-        gridsize = np.max((nobs, 512.))
-    gridsize = 2**np.ceil(np.log2(gridsize)) # round to next power of 2
+        gridsize = np.max((nobs, 512.0))
+    gridsize = 2 ** np.ceil(np.log2(gridsize))  # round to next power of 2
 
-    a = np.min(X) - cut * bw
-    b = np.max(X) + cut * bw
-    grid,delta = np.linspace(a, b, int(gridsize), retstep=True)
+    a = np.min(x) - cut * bw
+    b = np.max(x) + cut * bw
+    grid, delta = np.linspace(a, b, int(gridsize), retstep=True)
     RANGE = b - a
 
-#TODO: Fix this?
-# This is the Silverman binning function, but I believe it's buggy (SS)
-# weighting according to Silverman
-#    count = counts(X,grid)
-#    binned = np.zeros_like(grid)    #xi_{k} in Silverman
-#    j = 0
-#    for k in range(int(gridsize-1)):
-#        if count[k]>0: # there are points of X in the grid here
-#            Xingrid = X[j:j+count[k]] # get all these points
-#            # get weights at grid[k],grid[k+1]
-#            binned[k] += np.sum(grid[k+1]-Xingrid)
-#            binned[k+1] += np.sum(Xingrid-grid[k])
-#            j += count[k]
-#    binned /= (nobs)*delta**2 # normalize binned to sum to 1/delta
+    # TODO: Fix this?
+    # This is the Silverman binning function, but I believe it's buggy (SS)
+    # weighting according to Silverman
+    #    count = counts(x,grid)
+    #    binned = np.zeros_like(grid)    #xi_{k} in Silverman
+    #    j = 0
+    #    for k in range(int(gridsize-1)):
+    #        if count[k]>0: # there are points of x in the grid here
+    #            Xingrid = x[j:j+count[k]] # get all these points
+    #            # get weights at grid[k],grid[k+1]
+    #            binned[k] += np.sum(grid[k+1]-Xingrid)
+    #            binned[k+1] += np.sum(Xingrid-grid[k])
+    #            j += count[k]
+    #    binned /= (nobs)*delta**2 # normalize binned to sum to 1/delta
 
-#NOTE: THE ABOVE IS WRONG, JUST TRY WITH LINEAR BINNING
-    binned = fast_linbin(X, a, b, gridsize) / (delta * nobs)
+    # NOTE: THE ABOVE IS WRONG, JUST TRY WITH LINEAR BINNING
+    binned = fast_linbin(x, a, b, gridsize) / (delta * nobs)
 
     # step 2 compute FFT of the weights, using Munro (1976) FFT convention
     y = forrt(binned)
 
     # step 3 and 4 for optimal bw compute zstar and the density estimate f
-    # don't have to redo the above if just changing bw, ie., for cross val
+    # do not have to redo the above if just changing bw, ie., for cross val
 
-#NOTE: silverman_transform is the closed form solution of the FFT of the
-#gaussian kernel. Not yet sure how to generalize it.
-    zstar = silverman_transform(bw, gridsize, RANGE)*y # 3.49 in Silverman
-                                                    # 3.50 w Gaussian kernel
+    # NOTE: silverman_transform is the closed form solution of the FFT of the
+    # gaussian kernel. Not yet sure how to generalize it.
+    zstar = silverman_transform(bw, gridsize, RANGE) * y
+    # 3.49 in Silverman
+    # 3.50 w Gaussian kernel
     f = revrt(zstar)
     if retgrid:
         return f, grid, bw
     else:
         return f, bw
-
-if __name__ == "__main__":
-    import numpy as np
-    np.random.seed(12345)
-    xi = np.random.randn(100)
-    f,grid, bw1 = kdensity(xi, kernel="gau", bw=.372735, retgrid=True)
-    f2, bw2 = kdensityfft(xi, kernel="gau", bw="silverman",retgrid=False)
-
-# do some checking vs. silverman algo.
-# you need denes.f, http://lib.stat.cmu.edu/apstat/176
-#NOTE: I (SS) made some changes to the Fortran
-# and the FFT stuff from Munro http://lib.stat.cmu.edu/apstat/97o
-# then compile everything and link to denest with f2py
-#Make pyf file as usual, then compile shared object
-#f2py denest.f -m denest2 -h denest.pyf
-#edit pyf
-#-c flag makes it available to other programs, fPIC builds a shared library
-#/usr/bin/gfortran -Wall -c -fPIC fft.f
-#f2py -c denest.pyf ./fft.o denest.f
-
-    try:
-        from denest2 import denest # @UnresolvedImport
-        a = -3.4884382032045504
-        b = 4.3671504686785605
-        RANGE = b - a
-        bw = bandwidths.bw_silverman(xi)
-
-        ft,smooth,ifault,weights,smooth1 = denest(xi,a,b,bw,np.zeros(512),np.zeros(512),0,
-                np.zeros(512), np.zeros(512))
-# We use a different binning algo, so only accurate up to 3 decimal places
-        np.testing.assert_almost_equal(f2, smooth, 3)
-#NOTE: for debugging
-#        y2 = forrt(weights)
-#        RJ = np.arange(512/2+1)
-#        FAC1 = 2*(np.pi*bw/RANGE)**2
-#        RJFAC = RJ**2*FAC1
-#        BC = 1 - RJFAC/(6*(bw/((b-a)/M))**2)
-#        FAC = np.exp(-RJFAC)/BC
-#        SMOOTH = np.r_[FAC,FAC[1:-1]] * y2
-
-#        dens = revrt(SMOOTH)
-
-    except:
-#        ft = np.loadtxt('./ft_silver.csv')
-#        smooth = np.loadtxt('./smooth_silver.csv')
-        print("Didn't get the estimates from the Silverman algorithm")
